var cairounread = {
	MSG_FOLDER_FLAG_INBOX: 0x1000,
	onLoad : function(e) {
		dump("Loading Cairo Unread Count...\n");
		
		// read all the preferences
		const PREF_SERVICE = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		this.prefs = PREF_SERVICE.getBranch("extensions.cairo-dock-unread-messages@glxdock.org.");
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this.prefs.addObserver("", this, false);
     
     	this.traverseDeep = this.prefs.getBoolPref("traverse-deep");
     	this.animateType = this.prefs.getCharPref("animation-type");
     	this.animation = this.prefs.getCharPref("animation");
     	this.animateTime = this.prefs.getIntPref("animate-time");
	
		// initialization code
		this.initialized = true;
	},
	
	onClose: function(e) {
		dump("Closing Cairo Unread Count...\n");
		
		this.prefs.removeObserver("", this);
		
		this.initialized = true;
		this.resetUnreadCount();
	},
	
	resetUnreadCount: function() {
		dump("Resetting unread emblem\n");
		this.updateUnreadCount(0, true);
	},
	
	updateUnreadCount: function(x, blockingProcess){
		dump("Calling update count\n");
		dump("Finding path...\n");
		
		const DIR_SERVICE = new Components.Constructor("@mozilla.org/file/directory_service;1","nsIProperties");
		try { 
			path=(new DIR_SERVICE()).get("ProfD", Components.interfaces.nsIFile).path; 
		} catch (e) {
			alert(error);
		}
		
		path = path + "/extensions/cairo-dock-unread-messages@glxdock.org/chrome/content/cairo-SetQuickInfo.py";
		
		dump("Found path: " + path + "\n");

		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);		
		file.initWithPath("/usr/bin/env");
		
		var args = ["python", path, x];
		var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
		process.init(file);
		
		dump("Initialising process with arguments " + args + " is blocking = " + blockingProcess + "\n");
		process.run(blockingProcess, args, args.length);
	},
	updateUnreadAnimate: function(start){
		dump("Calling update count\n");
		dump("Finding path...\n");

		const DIR_SERVICE = new Components.Constructor("@mozilla.org/file/directory_service;1","nsIProperties");
		try { 
			path=(new DIR_SERVICE()).get("ProfD", Components.interfaces.nsIFile).path; 
		} catch (e) {
			alert(error);
		}

		path = path + "/extensions/cairo-dock-unread-messages@glxdock.org/chrome/content/cairo-Animate.py";

		dump("Found path: " + path + "\n");

		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);		
		file.initWithPath("/usr/bin/env");
		
		animateType = this.prefs.getCharPref("animation-type");
		if (start) {
			animateTime = this.prefs.getIntPref("animate-time");
			animation = this.prefs.getCharPref("animation");
		} else {
			animateTime = 0
			animation = ''
		}
		var args = ["python", path, animateTime, animation, animateType];
		var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
		process.init(file);

		dump("Initialising process with arguments " + args + "\n");
		process.run(false, args, args.length);  // false = non-blocking
	},

	onItemCountChanged : function() {
		dump("Item count changed...\n");
		if (this.timeoutId != -1) {
			window.clearTimeout(this.timeoutId);
		}
		// Schedule on the main thread
		this.timeoutId = window.setTimeout(this.performUnreadCount, 1000, this);
	},
	
	performUnreadCount: function(that) {
		dump("Counting unread messages...\n");
		var acctMgr = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
		var accounts = acctMgr.accounts;
		var totalCount = 0;
		dump("Found " + accounts.Count() + " accounts\n");
		for (var i = 0; i < accounts.Count(); i++) {
			var account = accounts.QueryElementAt(i, Components.interfaces.nsIMsgAccount);
			var rootFolder = account.incomingServer.rootFolder; // nsIMsgFolder            
				if (rootFolder.hasSubFolders) {
					totalCount += that.getTotalCount(rootFolder);
				}
		}
		dump("Found total : " + totalCount + "\n");
		that.updateUnreadCount(totalCount, false);
		// Add an animation if the count has gone up
		if (that.lastCount < totalCount){
		  that.updateUnreadAnimate(true);
		}
		else{
			that.updateUnreadAnimate(false);
		}
		that.lastCount = totalCount
	},

	getTotalCount: function(rootFolder) {
		if(rootFolder.getAllFoldersWithFlag) {
			return this._getTotalCountTB2(rootFolder);
		} else {
			return this._getTotalCountTB3(rootFolder);
		}
	},
	
	_getTotalCountTB2: function(rootFolder) {
		dump("Using _getTotalCountTB2\n");
		var totalCount = 0;
		dump("Finding all folders with inbox flag : " + this.MSG_FOLDER_FLAG_INBOX + "\n");
		var subFolders = rootFolder.getAllFoldersWithFlag(this.MSG_FOLDER_FLAG_INBOX); //nsISupportsArray
		dump("Found " + subFolders.Count() + "folders\n");
		
		for(var i = 0; i < subFolders.Count(); i++) {
			var folder = subFolders.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgFolder);
			dump("Get Number of unread messages with travese deep = " +  this.traverseDeep + "\n");
			totalCount += folder.getNumUnread(this.traverseDeep);
		}
		
		dump("Found total " + totalCount + "in all subFolders\n");
		return totalCount;
	},

	_getTotalCountTB3: function(rootFolder) {
		dump("Using _getTotalCountTB3\n");
		var totalCount = 0;
		dump("Finding all folders with inbox flag : " + this.MSG_FOLDER_FLAG_INBOX + "\n");
		var subFolders = rootFolder.getFoldersWithFlags(this.MSG_FOLDER_FLAG_INBOX); //nsIArray
		var subFoldersEnumerator = subFolders.enumerate();
		
		while(subFoldersEnumerator.hasMoreElements()) {
			var folder = subFoldersEnumerator.getNext().QueryInterface(Components.interfaces.nsIMsgFolder);
			dump("Get Number of unread messages with travese deep = " +  this.traverseDeep + "\n");
			totalCount += folder.getNumUnread(this.traverseDeep);
		}
		
		dump("Found total " + totalCount + "in all subFolders\n");
		return totalCount;
	},

	folderListener : {
		OnItemAdded : function(parent, item, viewString) {
				cairounread.onItemCountChanged();

		},
		OnItemRemoved : function(parent, item, viewString) {
				cairounread.onItemCountChanged();
		},
		OnItemPropertyFlagChanged : function(item, property, oldFlag, newFlag) {
			if (property=="Status"){
				cairounread.onItemCountChanged();
			}
		},
		OnItemEvent : function(item, event) {
				cairounread.onItemCountChanged();
		},
		
		OnFolderLoaded : function(aFolder) {},
		OnDeleteOrMoveMessagesCompleted : function(aFolder) {},
		OnItemPropertyChanged : function(parent, item, viewString) {},
		OnItemIntPropertyChanged : function(item, property, oldVal, newVal) {},
		OnItemBoolPropertyChanged : function(item, property, oldValue, newValue) {},
		OnItemUnicharPropertyChanged : function(item, property, oldValue, newValue) {}
	},
	
	observe: function(subject, topic, data) {
		if (topic != "nsPref:changed") {
			return;
		}
 
		switch(data) {
			case "traverse-deep":
				this.traverseDeep = this.prefs.getBoolPref("traverse-deep");
				cairounread.onItemCountChanged();
			break;
		}
	},
	
	mailSession: '',
	notifyFlags: '',
	timeoutId: -1
};

window.addEventListener("load", function(e) { cairounread.onLoad(e); }, false);
window.addEventListener("close", function(e) { cairounread.onClose(e); }, false); 
window.addEventListener("unload", function(e) { cairounread.onClose(e); }, false);
cairounread.mailSession = Components.classes["@mozilla.org/messenger/services/session;1"].getService(Components.interfaces.nsIMsgMailSession);
cairounread.notifyFlags = Components.interfaces.nsIFolderListener.all;
cairounread.mailSession.AddFolderListener(cairounread.folderListener, cairounread.notifyFlags);
